define("epi-addon-tinymce/plugins/epi-create-block/epi-create-block", [
    "epi-addon-tinymce/tinymce-loader",
    "epi-cms/widget/command/CreateContentFromSelector",
    "epi-addon-tinymce/ContentService",
    "epi-addon-tinymce/plugins/epi-dnd-processor/dndDropProcessor",
    "epi/i18n!epi/cms/nls/episerver.cms.tinymce.plugins.epicreateblock"
], function (
    tinymce,
    CreateContentFromSelector,
    ContentService,
    DndDropProcessor,
    pluginResource) {

    tinymce.PluginManager.add("epi-create-block", function (editor) {
        // We do not allow to create local blocks in Create mode because
        // we don't have the contextual folder (For this page/block) yet
        // that could be the parent for newly created local block
        if (editor.isInCreateMode || !editor.hasCreateAccessRights) {
            return;
        }

        var self = this;

        function mceEPiCreateBlock() {
            var command = new CreateContentFromSelector({
                creatingTypeIdentifier: "episerver.core.blockdata",
                createAsLocalAsset: true,
                isInQuickEditMode: editor.isInQuickEditMode,
                quickEditBlockId: editor._contentLink,
                keepCurrentView: editor.isEditing || self.isFullScreen,
                suppressOverlayAdjustments: self.isFullScreen,
                autoPublish: true
            });

            var contentService = new ContentService();

            command.set("model", {
                save: function (block) {
                    contentService.getContent(block.contentLink).then(function (model) {
                        var item = {
                            type: ["episerver.core.blockdata"],
                            data: model
                        };
                        var processor = new DndDropProcessor(editor);
                        processor.processData(item);
                    });
                }.bind(this),
                cancel: function () { }
            });
            command.execute();
        }

        editor.on("FullscreenStateChanged", function (event) {
            self.isFullScreen = event.state;
        });

        editor.ui.registry.addIcon("epi-iconSharedBlock", "<div class='epi-iconSharedBlock'></div>");

        // Register buttons
        editor.ui.registry.addButton("epi-create-block", {
            tooltip: pluginResource.title,
            onAction: mceEPiCreateBlock,
            icon: "epi-iconSharedBlock",
            onSetup: function (buttonApi) {
                function selectionChange() {
                    var isTextSelected = editor.selection.getContent() !== "";
                    buttonApi.setEnabled(!(editor.readonly || isTextSelected));
                }

                editor.on("SelectionChange", selectionChange);

                return function () {
                    editor.off("SelectionChange", selectionChange);
                };
            }
        });

        editor.shortcuts.add("ctrl+l", pluginResource.title, mceEPiCreateBlock);

        return {
            getMetadata: function () {
                return {
                    name: "Create block (optimizely)",
                    url: "https://www.optimizely.com/"
                };
            }
        };
    });
});
